#ifndef CHUNKREADER_H
#define CHUNKREADER_H

/*
   ChunkReader.h - simple reader for chunk format based files

   Copyright (C) 2003 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include "RefCounted.h"
#include "RCPtr.h"
#include "FileIO.h"

class ChunkReader;

class ChunkReader : public RefCounted {
public:
	static RCPtr<ChunkReader> OpenChunkFile(RCPtr<FileIO>& f);

	// open sub-chunk at current position and move
	// current position to the end of the sub-chunk
	RCPtr<ChunkReader> OpenChunk();

	inline const char* GetChunkName() const;
	inline size_t GetChunkLength() const;
	inline size_t GetCurrentPosition() const;

	bool ReadByte(unsigned char& byte);
	bool ReadWord(unsigned short& word);
	bool ReadDword(size_t &dword);
	bool ReadBlock(void* buf, size_t len);

	size_t CalculateCRC32() const;
	bool CalculateCRC32(size_t &checksum, size_t start_pos, size_t end_pos) const;

private:
	ChunkReader(RCPtr<FileIO>& f,
		size_t start,
		size_t end,
		const char* name=0);

	virtual ~ChunkReader();

	inline void SeekToCurrentPos() const;

private:
	RCPtr<FileIO> fFile;

	size_t fChunkStart; // absolute file position (included)
	size_t fChunkEnd; // absolute file position (not included)
	size_t fCurrentPosition; // relative position within this chunk

	char *fName;
};

inline const char* ChunkReader::GetChunkName() const
{
	return fName;
}

inline size_t ChunkReader::GetChunkLength() const
{
	return fChunkEnd-fChunkStart;
}

inline size_t ChunkReader::GetCurrentPosition() const
{
	return fCurrentPosition;
}

inline void ChunkReader::SeekToCurrentPos() const
{
	fFile->Seek(fChunkStart + fCurrentPosition);
}

#endif
